package dao

import (
	"contract_backend/contract/aon_executor"
	"contract_backend/contract/aon_user"
	"contract_backend/contract/erc20"
	"contract_backend/util"
	"crypto/ecdsa"
	"math/big"

	"github.com/ethereum/go-ethereum/accounts/abi"
	"github.com/ethereum/go-ethereum/accounts/abi/bind"
	"github.com/ethereum/go-ethereum/common"
	"github.com/ethereum/go-ethereum/crypto"
)

func (d *Dao) LoginByServer(byteKey []byte, userAddress, userId string) (txHash common.Hash, err error) {
	d.executorLock.Lock()
	defer d.executorLock.Unlock()
	instance, err := aon_user.NewAonUser(common.HexToAddress(d.c.Chain.UserContract), d.ethClient)
	if err != nil {
		return common.Hash{}, err
	}
	privateKey := crypto.ToECDSAUnsafe(byteKey)
	auth, err := bind.NewKeyedTransactorWithChainID(privateKey, d.chainId)
	if err != nil {
		return common.Hash{}, err
	}
	auth.GasFeeCap = d.gasPrice
	auth.GasTipCap = big.NewInt(1000000)
	auth.GasLimit = 700000
	tx, err := instance.LoginByServer(auth, common.HexToAddress(userAddress), userId, "")
	if err != nil {
		return common.Hash{}, err
	}
	_, err = d.WaitForReceipt(tx.Hash())
	return tx.Hash(), err
}

func (d *Dao) CheckPermit(userByteKey []byte) (txHash common.Hash, err error) {
	instance, err := erc20.NewErc20(common.HexToAddress(d.c.Chain.AppPointContract), d.ethClient)
	if err != nil {
		return
	}
	executePrivateKey := crypto.ToECDSAUnsafe(common.Hex2Bytes(d.c.Chain.ExecutorPrivateKey))
	userPrivateKey := crypto.ToECDSAUnsafe(userByteKey)
	userAddress := crypto.PubkeyToAddress(userPrivateKey.PublicKey)
	allowance, err := instance.Allowance(&bind.CallOpts{}, userAddress, common.HexToAddress(d.c.Chain.ExecutorContract))
	if err != nil {
		return
	}
	if allowance.Cmp(abi.MaxUint256) == 0 {
		return
	}

	// 执行permit
	auth, err := bind.NewKeyedTransactorWithChainID(executePrivateKey, d.chainId)
	if err != nil {
		return
	}
	auth.GasFeeCap = d.gasPrice
	auth.GasTipCap = big.NewInt(1000000)
	auth.GasLimit = 150000

	nextNonce, err := instance.Nonces(&bind.CallOpts{}, userAddress)
	if err != nil {
		return
	}

	v, r, s, err := util.PermitSign(
		userByteKey,
		"AON Application Score",
		"1",
		d.chainId,
		nextNonce,
		common.HexToAddress(d.c.Chain.AppPointContract),
		userAddress,
		common.HexToAddress(d.c.Chain.ExecutorContract))
	if err != nil {
		return
	}

	tx, err := instance.Permit(auth,
		userAddress,
		common.HexToAddress(d.c.Chain.ExecutorContract),
		abi.MaxUint256,
		big.NewInt(2000000000),
		v,
		r,
		s)
	if err != nil {
		return
	}
	return tx.Hash(), nil
}

func (d *Dao) ExecuteTask(privateKey *ecdsa.PrivateKey, executeId, userId, appId string, modelIds []string, price *big.Int) (txHash common.Hash, err error) {
	instance, err := aon_executor.NewAonExecutor(common.HexToAddress(d.c.Chain.ExecutorContract), d.ethClient)
	if err != nil {
		return common.Hash{}, err
	}

	auth, err := bind.NewKeyedTransactorWithChainID(privateKey, d.chainId)
	if err != nil {
		return common.Hash{}, err
	}
	auth.GasFeeCap = d.gasPrice
	auth.GasTipCap = big.NewInt(1000000)
	auth.GasLimit = 1500000

	tx, err := instance.ExecuteByUserId(auth, executeId, userId, appId, modelIds, price)
	if err != nil {
		return common.Hash{}, err
	}
	return tx.Hash(), nil
}
